@@ -1,6 +1,5 @@
 //! Program state processor
 
-use borsh::BorshSerialize;
 use solana_program::{
     account_info::{next_account_info, AccountInfo},
     entrypoint::ProgramResult,
@@ -17,7 +16,7 @@ use crate::{
         realm::get_realm_data,
         token_owner_record::{
             get_token_owner_record_address_seeds, get_token_owner_record_data_for_seeds,
-            TokenOwnerRecord,
+            TokenOwnerRecordV2,
         },
     },
     tools::spl_token::{get_spl_token_mint, get_spl_token_owner, transfer_spl_tokens},
@@ -41,8 +40,7 @@ pub fn process_deposit_governing_tokens(
     let system_info = next_account_info(account_info_iter)?; // 7
     let spl_token_info = next_account_info(account_info_iter)?; // 8
 
-    let rent_sysvar_info = next_account_info(account_info_iter)?; // 9
-    let rent = &Rent::from_account_info(rent_sysvar_info)?;
+    let rent = Rent::get()?;
 
     let realm_data = get_realm_data(program_id, realm_info)?;
     let governing_token_mint = get_spl_token_mint(governing_token_holding_info)?;
@@ -80,8 +78,8 @@ pub fn process_deposit_governing_tokens(
             return Err(GovernanceError::GoverningTokenOwnerMustSign.into());
         }
 
-        let token_owner_record_data = TokenOwnerRecord {
-            account_type: GovernanceAccountType::TokenOwnerRecord,
+        let token_owner_record_data = TokenOwnerRecordV2 {
+            account_type: GovernanceAccountType::TokenOwnerRecordV2,
             realm: *realm_info.key,
             governing_token_owner: *governing_token_owner_info.key,
             governing_token_deposit_amount: amount,
@@ -91,6 +89,7 @@ pub fn process_deposit_governing_tokens(
             total_votes_count: 0,
             outstanding_proposal_count: 0,
             reserved: [0; 7],
+            reserved_v2: [0; 128],
         };
 
         create_and_serialize_account_signed(
@@ -100,7 +99,7 @@ pub fn process_deposit_governing_tokens(
             &token_owner_record_address_seeds,
             program_id,
             system_info,
-            rent,
+            &rent,
         )?;
     } else {
         let mut token_owner_record_data = get_token_owner_record_data_for_seeds(
