@@ -106,7 +106,7 @@ async fn test_set_community_governance_delegate_with_owner_must_sign_error() {
 
     let hacker_governance_delegate = Keypair::new();
 
-    let mut instruction = set_governance_delegate(
+    let mut set_delegate_ix = set_governance_delegate(
         &governance_test.program_id,
         &token_owner_record_cookie.token_owner.pubkey(),
         &realm_cookie.address,
@@ -115,13 +115,13 @@ async fn test_set_community_governance_delegate_with_owner_must_sign_error() {
         &Some(hacker_governance_delegate.pubkey()),
     );
 
-    instruction.accounts[0] =
+    set_delegate_ix.accounts[0] =
         AccountMeta::new_readonly(token_owner_record_cookie.token_owner.pubkey(), false);
 
     // Act
     let err = governance_test
         .bench
-        .process_transaction(&[instruction], None)
+        .process_transaction(&[set_delegate_ix], None)
         .await
         .err()
         .unwrap();
